/*
 * Copyright 2009 Nicolas LE GRIEL.
 * 
 * Les sources qui constituent ce projet nlegriel-dvp-defi5 de mme que 
 * la documentation associe sont la proprit de leur auteur.
 * 
 * Je donne mon accord au site developpez.com pour l'utilisation de 
 * tout ou partie des sources et de la documentation de ce projet dans 
 * les pages developpez.com
 */
package com.developpez.nlegriel.defi5;

/**
 * Reprsente un cube du puzzle par ses coordonnes 3D.
 *
 * @author Nicolas LE GRIEL
 */
public class Cube implements Comparable<Cube> {
	public int x;
	public int y;
	public int z;
	
	public Cube() {
	}
	
	public Cube(int x, int y, int z) {
		this.x = x;
		this.y = y;
		this.z = z;
	}

	@Override
	public boolean equals(Object o) {
		if (o instanceof Cube) {
			Cube p = (Cube) o;
			return p.x == this.x && p.y == this.y && p.z == this.z;
		}
		return false;
	}
	
	@Override
	public int hashCode() {
		return 100*x + 10*y + z;
	}
	
	/**
	 * Affiche les coordonnes du cube.
	 */
	@Override
	public String toString() {
		return "("+x+", "+y+", "+z+")";
	}

	/**
	 * Compare les cubes du puzzle par leurs coordonnes dans l'espace dans
	 * l'ordre abscisses, ordonnes, cotes.
	 */
	@Override
	public int compareTo(Cube c) {
		if (x != c.x)
			return x - c.x;
		else if (y != c.y)
			return y - c.y;
		else
			return z - c.z;
	}

	/**
	 * Se sert des coordonnes d'un autre cube pour pouvoir dterminer
	 * les coordones du cube courant translat par un vecteur ayant les
	 * coordonnes du cube. Il ne s'agit en fait que d'une "somme" de cubes
	 * (les coordonnes sont additionnes).
	 * 
	 * @param c contient les informations de la "translation"
	 * 
	 * @return un nouvel objet de type {@link Cube} dont les coordonnes sont
	 * les sommes des coordonnes du cube courant et du cube fourni en argument
	 */
	public Cube translate(Cube c) {
		return new Cube(x + c.x, y + c.y, z + c.z);
	}
}
