/*
 * Copyright 2009 Nicolas LE GRIEL.
 * 
 * Les sources qui constituent ce projet nlegriel-dvp-defi5 de mme que 
 * la documentation associe sont la proprit de leur auteur.
 * 
 * Je donne mon accord au site developpez.com pour l'utilisation de 
 * tout ou partie des sources et de la documentation de ce projet dans 
 * les pages developpez.com
 */
package com.developpez.nlegriel.defi5.view;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.FileNotFoundException;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;

import com.developpez.nlegriel.defi5.Launcher;

/**
 * Fentre utilise pour le mode d'excution visuelle de l'application.
 *
 * @author Nicolas LE GRIEL
 */
public class LauncherWindow extends JFrame {
	private static final long serialVersionUID = 1L;

	private Launcher launcher;

	private JPanel contentPanel;
	
	private JLabel inputFileLabel;
	private JTextField inputFileTextField;
	private JButton inputFileChooseButton;
	private JButton seePuzzleButton;
	private JButton executeButton;
	
	private JPanel statusPanel;
	
	private JTextArea outputArea;
	
	private JButton seeSolutionButton;
	private JButton quitButton;
	
	
	public LauncherWindow() {
		super();
	}
	
	public LauncherWindow(Launcher launcher) {
		super();
		this.launcher = launcher;
		initialize();
	}

	/**
	 * Gre toute la construction des contrles de la fentre.
	 */
	private void initialize() {
		setTitle("Cube de SOMA - Indiquer un fichier de description de puzzle");
		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		setContentPane(getContentPanel());
		pack();
		setLocation((Toolkit.getDefaultToolkit().getScreenSize().width - getWidth()) / 2, (Toolkit.getDefaultToolkit().getScreenSize().height - getHeight()) / 2);
	}
	
	/**
	 * Gre le placement des lments graphiques de la fentre.
	 */
	public JPanel getContentPanel() {
		if (contentPanel == null) {
			contentPanel = new JPanel();
			GridBagLayout contentLayout = new GridBagLayout();
			contentPanel.setLayout(contentLayout);
			GridBagConstraints constraints = new GridBagConstraints();
			
			constraints.gridx = 0;
			constraints.gridy = 0;
			constraints.gridheight = 1;
			constraints.gridwidth = 1;
			constraints.anchor = GridBagConstraints.PAGE_START;
			constraints.fill = GridBagConstraints.BOTH;
			constraints.insets = new Insets(5, 5, 5, 5);
			contentPanel.add(getInputFileLabel(), constraints);
			
			constraints.gridx = 1;
			constraints.gridwidth = 2;
			contentPanel.add(getInputFileTextField(), constraints);
			
			constraints.gridx = 3;
			constraints.gridwidth = 1;
			constraints.anchor = GridBagConstraints.FIRST_LINE_START;
			constraints.fill = GridBagConstraints.NONE;
			contentPanel.add(getInputFileChooseButton(), constraints);

			constraints.gridx = 1;
			constraints.gridy = 1;
			constraints.anchor = GridBagConstraints.PAGE_START;
			contentPanel.add(getSeePuzzleButton(), constraints);

			constraints.gridx = 2;
			contentPanel.add(getExecuteButton(), constraints);

			constraints.gridx = 0;
			constraints.gridy = 2;
			constraints.gridwidth = 4;
			constraints.fill = GridBagConstraints.BOTH;
			contentPanel.add(getStatusPanel(), constraints);

			constraints.gridx = 2;
			constraints.gridy = 3;
			constraints.gridwidth = 1;
			constraints.fill = GridBagConstraints.NONE;
			contentPanel.add(getSeeSolutionButton(), constraints);

			constraints.gridx = 3;
			contentPanel.add(getQuitButton(), constraints);
		}
		return contentPanel;
	}

	public JLabel getInputFileLabel() {
		if (inputFileLabel == null) {
			inputFileLabel = new JLabel();
			inputFileLabel.setText("Dfinition du puzzle :");
			inputFileLabel.setHorizontalAlignment(JLabel.RIGHT);
		}
		return inputFileLabel;
	}

	public JTextField getInputFileTextField() {
		if (inputFileTextField == null) {
			inputFileTextField = new JTextField();
			inputFileTextField.setColumns(30);
			inputFileTextField.setEditable(false);
		}
		return inputFileTextField;
	}

	public JButton getInputFileChooseButton() {
		if (inputFileChooseButton == null) {
			inputFileChooseButton = new JButton();
			inputFileChooseButton.setText("<html><b>Rechercher le fichier...</b></html>");
			inputFileChooseButton.addActionListener(new ActionListener() {
				@Override
				public void actionPerformed(ActionEvent event) {
					JFileChooser chooser = new JFileChooser(".");
					chooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
					File file = new File(getInputFileTextField().getText());
					chooser.setSelectedFile(file);
					chooser.showOpenDialog(null);
					if (chooser.getSelectedFile() != null && chooser.getSelectedFile().isFile()) {
						getInputFileTextField().setText(chooser.getSelectedFile().getAbsolutePath());
						boolean isEnabled = getInputFileTextField().getText() != null && ! getInputFileTextField().getText().isEmpty();
						getExecuteButton().setEnabled(isEnabled);
						getSeePuzzleButton().setEnabled(isEnabled);
						getSeeSolutionButton().setEnabled(false);
						getOutputArea().setText("");
						try {
							launcher = new Launcher(getInputFileTextField().getText());
						}
						catch (FileNotFoundException e) {
							JOptionPane.showMessageDialog(null, "Le fichier "+getInputFileTextField()+" n'a pas pu tre ouvert", "Erreur", JOptionPane.ERROR_MESSAGE);
						}
						catch (Exception e) {
							JOptionPane.showMessageDialog(null, e.getMessage(), "Erreur", JOptionPane.ERROR_MESSAGE);
						}
					}
				}
			});
		}
		return inputFileChooseButton;
	}

	public JButton getSeePuzzleButton() {
		if (seePuzzleButton == null) {
			seePuzzleButton = new JButton();
			seePuzzleButton.setText("Visualiser le puzzle");
			seePuzzleButton.setEnabled(false);
			seePuzzleButton.addActionListener(new ActionListener() {
				@Override
				public void actionPerformed(ActionEvent event) {
					try {
						launcher.initPuzzle();
						new CubesViewWindow("Visualisation de la figure  raliser", launcher.getPuzzle());
						getExecuteButton().setEnabled(true);
						getSeePuzzleButton().setEnabled(true);
					}
					catch (Exception e) {
						JOptionPane.showMessageDialog(null, "Une erreur s'est produite durant la lecture du fichier : "+e.getMessage(), "Erreur", JOptionPane.ERROR_MESSAGE);
						getExecuteButton().setEnabled(false);
						getSeePuzzleButton().setEnabled(false);
					}
				}
			});
		}
		return seePuzzleButton;
	}

	public JButton getExecuteButton() {
		if (executeButton == null) {
			executeButton = new JButton();
			executeButton.setText("Rechercher une solution");
			executeButton.setBackground(new Color(0, 210, 0));
			executeButton.addActionListener(new ActionListener() {
				@Override
				public void actionPerformed(ActionEvent event) {
					try {
						launcher.execute();
					}
					catch (Exception e) {
						e.printStackTrace(launcher.getWriter());
						JOptionPane.showMessageDialog(null, e.getMessage(), "Erreur", JOptionPane.ERROR_MESSAGE);
					}
					getSeePuzzleButton().setEnabled(false);
					getExecuteButton().setEnabled(false);
					getOutputArea().setText(launcher.getStringWriter().toString());
					getSeeSolutionButton().setEnabled(launcher.getStatus() == Launcher.Status.SUCCESS);
					pack();
				}
			});
			executeButton.setEnabled(getInputFileTextField().getText() != null && ! getInputFileTextField().getText().isEmpty());
		}
		return executeButton;
	}

	public JPanel getStatusPanel() {
		if (statusPanel == null) {
			statusPanel = new JPanel();
			statusPanel.setBorder(
					BorderFactory.createCompoundBorder(
							BorderFactory.createTitledBorder("Rsultat"),
							BorderFactory.createEmptyBorder(5,5,5,5)));

			statusPanel.setLayout(new BorderLayout(5, 5));
			JScrollPane statusScrollPane = new JScrollPane();
			statusScrollPane.setViewportView(getOutputArea());
			statusPanel.add(statusScrollPane, BorderLayout.CENTER);
		}
		return statusPanel;
	}

	public JTextArea getOutputArea() {
		if (outputArea == null) {
			outputArea = new JTextArea();
			outputArea.setRows(10);
			outputArea.setColumns(80);
			outputArea.setEditable(false);
		}
		return outputArea;
	}

	public JButton getSeeSolutionButton() {
		if (seeSolutionButton == null) {
			seeSolutionButton = new JButton();
			seeSolutionButton.setText("Visualiser la solution");
			seeSolutionButton.setEnabled(false);
			seeSolutionButton.addActionListener(new ActionListener() {
				@Override
				public void actionPerformed(ActionEvent e) {
					new CubesViewWindow("Voici UNE solution du puzzle", launcher.getPuzzle());
				}
			});
		}
		return seeSolutionButton;
	}

	public JButton getQuitButton() {
		if (quitButton == null) {
			quitButton = new JButton();
			quitButton.setText("Quitter l'application");
			quitButton.addActionListener(new ActionListener() {
				@Override
				public void actionPerformed(ActionEvent e) {
					dispose();
					System.exit(0);
				}
			});
		}
		return quitButton;
	}
}
