/*
 * Copyright 2009 Nicolas LE GRIEL.
 * 
 * Les sources qui constituent ce projet nlegriel-dvp-defi5 de mme que 
 * la documentation associe sont la proprit de leur auteur.
 * 
 * Je donne mon accord au site developpez.com pour l'utilisation de 
 * tout ou partie des sources et de la documentation de ce projet dans 
 * les pages developpez.com
 */
package com.developpez.nlegriel.defi5;

import static org.junit.Assert.assertEquals;

import java.util.Scanner;

import org.junit.Test;

import com.developpez.nlegriel.defi5.Cube;
import com.developpez.nlegriel.defi5.Puzzle;

/**
 * Valide le fonctionnement de la reprsentation d'un
 * puzzle.
 *
 * @author Nicolas LE GRIEL
 */
public class PuzzleTest {
	@Test(expected=IllegalArgumentException.class)
	public void loadFromScanner_MissingLines() {
		Scanner scanner = new Scanner(this.getClass().getResourceAsStream("/SOMA_manqueLignes.txt"));
		new Puzzle().loadFromScanner(scanner);
	}

	@Test(expected=IllegalArgumentException.class)
	public void loadFromScanner_TooMuchLines() {
		Scanner scanner = new Scanner(this.getClass().getResourceAsStream("/SOMA_lignesEnTrop.txt"));
		new Puzzle().loadFromScanner(scanner);
	}

	@Test(expected=IllegalArgumentException.class)
	public void loadFromScanner_NotANumber() {
		Scanner scanner = new Scanner(this.getClass().getResourceAsStream("/SOMA_lettre.txt"));
		new Puzzle().loadFromScanner(scanner);
	}

	@Test(expected=IllegalArgumentException.class)
	public void loadFromScanner_TooBigNumber() {
		Scanner scanner = new Scanner(this.getClass().getResourceAsStream("/SOMA_coordonneeIllegale.txt"));
		new Puzzle().loadFromScanner(scanner);
	}

	@Test(expected=IllegalArgumentException.class)
	public void loadFromScanner_BadSeparator() {
		Scanner scanner = new Scanner(this.getClass().getResourceAsStream("/SOMA_mauvaiSeparateur.txt"));
		new Puzzle().loadFromScanner(scanner);
	}

	@Test(expected=IllegalArgumentException.class)
	public void loadFromScanner_IdenticalCubes() {
		Scanner scanner = new Scanner(this.getClass().getResourceAsStream("/SOMA_ligneEnDouble.txt"));
		new Puzzle().loadFromScanner(scanner);
	}
	
	@Test
	public void loadFromScanner_Ok() {
		Scanner scanner = new Scanner(this.getClass().getResourceAsStream("/SOMA_Ok.txt"));
		Puzzle puzzle = new Puzzle();
		puzzle.loadFromScanner(scanner);
		Cube cube = new Cube();
		cube.x = 1;
		assertEquals(cube, puzzle.cubes[1]);
		cube = new Cube();
		cube.x = 2;
		cube.z = 1;
		assertEquals(cube, puzzle.cubes[3]);
		cube = new Cube();
		cube.x = 6;
		cube.y = 2;
		cube.z = 2;
		assertEquals(cube, puzzle.cubes[26]);
	}
}
